@@ -1,57 +1,53 @@
 'use strict';
 
-/*eslint key-spacing: 0, comma-spacing: 0, quotes: 0*/
+/*eslint key-spacing: 0, comma-spacing: 0 */
 
-var rbush = require('../rbush.js'),
+var rbush = require('..'),
     t = require('tape');
 
 function sortedEqual(t, a, b, compare) {
+    compare = compare || defaultCompare;
     t.same(a.slice().sort(compare), b.slice().sort(compare));
 }
 
+function defaultCompare(a, b) {
+    return (a.minX - b.minX) || (a.minY - b.minY) || (a.maxX - b.maxX) || (a.maxY - b.maxY);
+}
+
 function someData(n) {
     var data = [];
 
     for (var i = 0; i < n; i++) {
-        data.push([i, i, i, i]);
+        data.push({minX: i, minY: i, maxX: i, maxY: i});
     }
     return data;
 }
 
+function arrToBBox(arr) {
+    return {
+        minX: arr[0],
+        minY: arr[1],
+        maxX: arr[2],
+        maxY: arr[3]
+    };
+}
+
 var data = [[0,0,0,0],[10,10,10,10],[20,20,20,20],[25,0,25,0],[35,10,35,10],[45,20,45,20],[0,25,0,25],[10,35,10,35],
     [20,45,20,45],[25,25,25,25],[35,35,35,35],[45,45,45,45],[50,0,50,0],[60,10,60,10],[70,20,70,20],[75,0,75,0],
     [85,10,85,10],[95,20,95,20],[50,25,50,25],[60,35,60,35],[70,45,70,45],[75,25,75,25],[85,35,85,35],[95,45,95,45],
     [0,50,0,50],[10,60,10,60],[20,70,20,70],[25,50,25,50],[35,60,35,60],[45,70,45,70],[0,75,0,75],[10,85,10,85],
     [20,95,20,95],[25,75,25,75],[35,85,35,85],[45,95,45,95],[50,50,50,50],[60,60,60,60],[70,70,70,70],[75,50,75,50],
-    [85,60,85,60],[95,70,95,70],[50,75,50,75],[60,85,60,85],[70,95,70,95],[75,75,75,75],[85,85,85,85],[95,95,95,95]];
-
-var testTree = {"children":[{
-    "children":[
-        {"children":[[0,0,0,0],[10,10,10,10],[20,20,20,20],[25,0,25,0]],"height":1,"bbox":[0,0,25,20],"leaf":true},
-        {"children":[[0,25,0,25],[10,35,10,35],[25,25,25,25],[20,45,20,45]],"height":1,"bbox":[0,25,25,45],"leaf":true},
-        {"children":[[50,0,50,0],[35,10,35,10],[60,10,60,10],[45,20,45,20]],"height":1,"bbox":[35,0,60,20],"leaf":true},
-        {"children":[[50,25,50,25],[35,35,35,35],[60,35,60,35],[45,45,45,45]],"height":1,"bbox":[35,25,60,45],"leaf":true}
-    ],"height":2,"bbox":[0,0,60,45]
-},{
-    "children":[
-        {"children":[[0,50,0,50],[25,50,25,50],[10,60,10,60],[20,70,20,70]],"height":1,"bbox":[0,50,25,70],"leaf":true},
-        {"children":[[0,75,0,75],[25,75,25,75],[10,85,10,85],[20,95,20,95]],"height":1,"bbox":[0,75,25,95],"leaf":true},
-        {"children":[[35,60,35,60],[50,50,50,50],[60,60,60,60],[45,70,45,70]],"height":1,"bbox":[35,50,60,70],"leaf":true},
-        {"children":[[50,75,50,75],[60,85,60,85],[45,95,45,95],[35,85,35,85]],"height":1,"bbox":[35,75,60,95],"leaf":true}
-    ],"height":2,"bbox":[0,50,60,95]
-},{
-    "children":[
-        {"children":[[70,20,70,20],[75,0,75,0],[75,25,75,25],[70,45,70,45]],"height":1,"bbox":[70,0,75,45],"leaf":true},
-        {"children":[[75,50,75,50],[70,70,70,70],[75,75,75,75],[70,95,70,95]],"height":1,"bbox":[70,50,75,95],"leaf":true},
-        {"children":[[85,10,85,10],[95,20,95,20],[85,35,85,35],[95,45,95,45]],"height":1,"bbox":[85,10,95,45],"leaf":true},
-        {"children":[[85,60,85,60],[95,70,95,70],[85,85,85,85],[95,95,95,95]],"height":1,"bbox":[85,60,95,95],"leaf":true}
-    ],"height":2,"bbox":[70,0,95,95]
-}],"height":3,"bbox":[0,0,95,95]};
+    [85,60,85,60],[95,70,95,70],[50,75,50,75],[60,85,60,85],[70,95,70,95],[75,75,75,75],[85,85,85,85],[95,95,95,95]]
+    .map(arrToBBox);
 
+var emptyData = [[-Infinity, -Infinity, Infinity, Infinity],[-Infinity, -Infinity, Infinity, Infinity],
+    [-Infinity, -Infinity, Infinity, Infinity],[-Infinity, -Infinity, Infinity, Infinity],
+    [-Infinity, -Infinity, Infinity, Infinity],[-Infinity, -Infinity, Infinity, Infinity]].map(arrToBBox);
 
 t('constructor accepts a format argument to customize the data format', function (t) {
     var tree = rbush(4, ['.minLng', '.minLat', '.maxLng', '.maxLat']);
-    t.same(tree.toBBox({minLng: 1, minLat: 2, maxLng: 3, maxLat: 4}), [1, 2, 3, 4]);
+    t.same(tree.toBBox({minLng: 1, minLat: 2, maxLng: 3, maxLat: 4}),
+        {minX: 1, minY: 2, maxX: 3, maxY: 4});
     t.end();
 });
 
@@ -68,7 +64,12 @@ t('#toBBox, #compareMinX, #compareMinY can be overriden to allow custom data str
 
     var tree = rbush(4);
     tree.toBBox = function (item) {
-        return [item.minLng, item.minLat, item.maxLng, item.maxLat];
+        return {
+            minX: item.minLng,
+            minY: item.minLat,
+            maxX: item.maxLng,
+            maxY: item.maxLat
+        };
     };
     tree.compareMinX = function (a, b) {
         return a.minLng - b.minLng;
@@ -90,29 +91,29 @@ t('#toBBox, #compareMinX, #compareMinY can be overriden to allow custom data str
         return a.minLng - b.minLng || a.minLat - b.minLat;
     }
 
-    sortedEqual(t, tree.search([-180, -90, 180, 90]), [
+    sortedEqual(t, tree.search({minX: -180, minY: -90, maxX: 180, maxY: 90}), [
         {minLng: -115, minLat:  45, maxLng: -105, maxLat:  55},
         {minLng:  105, minLat:  45, maxLng:  115, maxLat:  55},
         {minLng:  105, minLat: -55, maxLng:  115, maxLat: -45},
         {minLng: -115, minLat: -55, maxLng: -105, maxLat: -45}
     ], byLngLat);
 
-    sortedEqual(t, tree.search([-180, -90, 0, 90]), [
+    sortedEqual(t, tree.search({minX: -180, minY: -90, maxX: 0, maxY: 90}), [
         {minLng: -115, minLat:  45, maxLng: -105, maxLat:  55},
         {minLng: -115, minLat: -55, maxLng: -105, maxLat: -45}
     ], byLngLat);
 
-    sortedEqual(t, tree.search([0, -90, 180, 90]), [
+    sortedEqual(t, tree.search({minX: 0, minY: -90, maxX: 180, maxY: 90}), [
         {minLng: 105, minLat:  45, maxLng: 115, maxLat:  55},
         {minLng: 105, minLat: -55, maxLng: 115, maxLat: -45}
     ], byLngLat);
 
-    sortedEqual(t, tree.search([-180, 0, 180, 90]), [
+    sortedEqual(t, tree.search({minX: -180, minY: 0, maxX: 180, maxY: 90}), [
         {minLng: -115, minLat: 45, maxLng: -105, maxLat: 55},
         {minLng:  105, minLat: 45, maxLng:  115, maxLat: 55}
     ], byLngLat);
 
-    sortedEqual(t, tree.search([-180, -90, 180, 0]), [
+    sortedEqual(t, tree.search({minX: -180, minY: -90, maxX: 180, maxY: 0}), [
         {minLng:  105, minLat: -55, maxLng:  115, maxLat: -45},
         {minLng: -115, minLat: -55, maxLng: -105, maxLat: -45}
     ], byLngLat);
@@ -123,7 +124,7 @@ t('#toBBox, #compareMinX, #compareMinY can be overriden to allow custom data str
 t('#load bulk-loads the given data given max node entries and forms a proper search tree', function (t) {
 
     var tree = rbush(4).load(data);
-    sortedEqual(t, tree.all(), rbush(4).fromJSON(testTree).all());
+    sortedEqual(t, tree.all(), data);
 
     t.end();
 });
@@ -151,6 +152,29 @@ t('#load does nothing if loading empty data', function (t) {
     t.end();
 });
 
+t('#load handles the insertion of maxEntries + 2 empty bboxes', function (t) {
+    var tree = rbush(4)
+        .load(emptyData);
+
+    t.equal(tree.toJSON().height, 2);
+    sortedEqual(t, tree.all(), emptyData);
+
+    t.end();
+});
+
+t('#insert handles the insertion of maxEntries + 2 empty bboxes', function (t) {
+    var tree = rbush(4);
+
+    emptyData.forEach(function (datum) {
+        tree.insert(datum);
+    });
+
+    t.equal(tree.toJSON().height, 2);
+    sortedEqual(t, tree.all(), emptyData);
+
+    t.end();
+});
+
 t('#load properly splits tree root when merging trees of the same height', function (t) {
     var tree = rbush(4)
         .load(data)
@@ -184,12 +208,12 @@ t('#load properly merges data of smaller or bigger tree heights', function (t) {
 t('#search finds matching points in the tree given a bbox', function (t) {
 
     var tree = rbush(4).load(data);
-    var result = tree.search([40, 20, 80, 70]);
+    var result = tree.search({minX: 40, minY: 20, maxX: 80, maxY: 70});
 
     sortedEqual(t, result, [
         [70,20,70,20],[75,25,75,25],[45,45,45,45],[50,50,50,50],[60,60,60,60],[70,70,70,70],
         [45,20,45,20],[45,70,45,70],[75,50,75,50],[50,25,50,25],[60,35,60,35],[70,45,70,45]
-    ]);
+    ].map(arrToBBox));
 
     t.end();
 });
@@ -197,9 +221,9 @@ t('#search finds matching points in the tree given a bbox', function (t) {
 t('#collides returns true when search finds matching points', function (t) {
 
     var tree = rbush(4).load(data);
-    var result = tree.collides([40, 20, 80, 70]);
+    var result = tree.collides({minX: 40, minY: 20, maxX: 80, maxY: 70});
 
-	t.same(result, true);
+    t.same(result, true);
 
     t.end();
 });
@@ -224,48 +248,39 @@ t('#all returns all points in the tree', function (t) {
     var result = tree.all();
 
     sortedEqual(t, result, data);
-    sortedEqual(t, tree.search([0, 0, 100, 100]), data);
+    sortedEqual(t, tree.search({minX: 0, minY: 0, maxX: 100, maxY: 100}), data);
 
     t.end();
 });
 
 t('#toJSON & #fromJSON exports and imports search tree in JSON format', function (t) {
 
-    var tree = rbush(4);
-    tree.fromJSON(testTree);
-
-    var tree2 = rbush(4).load(data);
+    var tree = rbush(4).load(data);
+    var tree2 = rbush(4).fromJSON(tree.data);
 
     sortedEqual(t, tree.all(), tree2.all());
     t.end();
 });
 
 t('#insert adds an item to an existing tree correctly', function (t) {
-    var tree = rbush(4).load([
+    var items = [
         [0, 0, 0, 0],
         [1, 1, 1, 1],
-        [2, 2, 2, 2]
-    ]);
-    tree.insert([3, 3, 3, 3]);
+        [2, 2, 2, 2],
+        [3, 3, 3, 3],
+        [1, 1, 2, 2]
+    ].map(arrToBBox);
 
-    t.same(tree.toJSON(), {
-        'children':[[0,0,0,0],[1,1,1,1],[2,2,2,2],[3,3,3,3]],
-        'leaf':true,
-        'height':1,
-        'bbox':[0,0,3,3]
-    });
+    var tree = rbush(4).load(items.slice(0, 3));
 
-    tree.insert([1, 1, 2, 2]);
+    tree.insert(items[3]);
+    t.equal(tree.toJSON().height, 1);
+    sortedEqual(t, tree.all(), items.slice(0, 4));
+
+    tree.insert(items[4]);
+    t.equal(tree.toJSON().height, 2);
+    sortedEqual(t, tree.all(), items);
 
-    t.same(tree.toJSON(), {
-        'children':[
-            {'children':[[0,0,0,0],[1,1,1,1]],'leaf':true,'height':1,'bbox':[0,0,1,1]},
-            {'children':[[1,1,2,2],[2,2,2,2],[3,3,3,3]],'height':1,'leaf':true,'bbox':[1,1,3,3]}
-        ],
-        'height':2,
-        'bbox':[0,0,3,3],
-        'leaf':false
-    });
     t.end();
 });
 
@@ -331,6 +346,19 @@ t('#remove brings the tree to a clear state when removing everything one by one'
     t.same(tree.toJSON(), rbush(4).toJSON());
     t.end();
 });
+t('#remove accepts an equals function', function (t) {
+    var tree = rbush(4).load(data);
+
+    var item = {minX: 20, minY: 70, maxX: 20, maxY: 70, foo: 'bar'};
+
+    tree.insert(item);
+    tree.remove(JSON.parse(JSON.stringify(item)), function (a, b) {
+        return a.foo === b.foo;
+    });
+
+    sortedEqual(t, tree.all(), data);
+    t.end();
+});
 
 t('#clear should clear all the data in the tree', function (t) {
     t.same(
@@ -344,8 +372,7 @@ t('should have chainable API', function (t) {
         rbush()
             .load(data)
             .insert(data[0])
-            .remove(data[0])
-            .fromJSON(testTree);
+            .remove(data[0]);
     });
     t.end();
 });
